from sqlalchemy import *

from sqlalchemy.ext.sessioncontext import SessionContext
from sqlalchemy.ext.assignmapper import assign_mapper

__all__ = ['meta',
           'ctx',
           'machine_table',
           'nic_table',
           'disk_table',
           'types_table',
           'cdroms_table',
           'Machine',
           'NIC',
           'Disk',
           'Type',
           'CDROM']

meta = DynamicMetaData()
ctx = SessionContext(create_session)

machine_table = Table('machines', meta,
       Column('machine_id', Integer, primary_key=True, nullable=False),
       Column('name', String, nullable=False),
       Column('memory', Integer, nullable=False),
       Column('owner', String, nullable=False),
       Column('contact', String, nullable=False),
       Column('uuid', String, nullable=False),
       Column('administrator', String, nullable=False, default=False),
       Column('type_id', String, ForeignKey('types.type_id'), nullable=False),
       Column('autorestart', Boolean, nullable=False, default=False),
       Column('cpus', Integer, nullable=False, default=1))

nic_table = Table('nics', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id'), nullable=True),
       Column('mac_addr', String, nullable=False, primary_key=True),
       Column('ip', String, nullable=False, unique=True),
       Column('hostname', String, nullable=True))

disk_table = Table('disks', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id'), nullable=False),
       Column('guest_device_name', String, nullable=False),
       Column('size', Integer, nullable=False),
       PrimaryKeyConstraint('machine_id', 'guest_device_name'))

types_table = Table('types', meta,
       Column('type_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False),
       Column('hvm', Boolean, nullable=False),
       Column('apic', Boolean, nullable=False),
       Column('acpi', Boolean, nullable=False),
       Column('pae', Boolean, nullable=False))

cdroms_table = Table('cdroms', meta,
       Column('cdrom_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False))


class Machine(object):
    pass

class NIC(object):
    def __init__(self, machine_id, mac_addr, ip, hostname):
        self.machine_id = machine_id
        self.mac_addr = mac_addr
        self.ip = ip
        self.hostname = hostname

class Disk(object):
    def __init__(self, machine_id, guest, size):
        self.machine_id = machine_id
        self.guest_device_name = guest
        self.size = size

class Type(object):
    pass

class CDROM(object):
    def __init__(self, cdrom_id, description):
        self.cdrom_id = cdrom_id
        self.description = description

assign_mapper(ctx, Machine, machine_table,
              properties={'nics': relation(NIC, backref="machine"),
                          'disks': relation(Disk, backref="machine"),
                          'type': relation(Type)});
assign_mapper(ctx, NIC, nic_table)
assign_mapper(ctx, Disk, disk_table)
assign_mapper(ctx, Type, types_table)
assign_mapper(ctx, CDROM, cdroms_table)

